//////////////////////////////////////////////////////
/// The following example illustrates a COM+ type
/// developed using C# and the .NET Framework.
/// 
/// author: Gopalan Suresh Raj
/// Copyright (c), 2001. All Rights Reserved.
/// URL: http://gsraj.tripod.com/
/// email: gopalan@gmx.net
/// 
/// <register>
/// gacutil /i Bank.dll
/// regsvcs /fc Bank.dll
/// </register>
//////////////////////////////////////////////////////

using System;
using System.Runtime.InteropServices;
using System.EnterpriseServices;
// Include the following for the Trace class
using System.Diagnostics;   
// Include the following for Windows Message Box
using System.Windows.Forms; 
// Include the BookKeeper namespace
using BookKeeper;

namespace Bank {

  ///////////////////////////////////////////////////////////////
  /// <summary>
  /// The Account Manager interface
  /// </summary>
  /// <remarks>
  /// This interface defines create and delete methods to
  /// add or delete Bank Accounts
  /// </remarks>
  ///////////////////////////////////////////////////////////////
  
  /// Indicate whether a managed interface is dual, IDispatch or 
  /// IUnknown based when exposed to COM
  [ InterfaceTypeAttribute( ComInterfaceType.InterfaceIsDual ) ]

  public interface IAccountManager {

    /// <summary>
    /// The create method
    /// </summary>
    /// <remarks>
    /// Method used to create a new Bank account
    /// </remarks>
    /// <param name="type">Either Checking or Savings</param>
    /// <param name="customerNames">Customers who own this account</param>
    /// <param name="startingBalance">Initial Deposit</param>
    /// <returns></returns>
    int create (AccountType type, string[] customerNames, float startingBalance);

    /// <summary>
    /// The delete method
    /// </summary>
    /// <remarks>
    /// Method used to delete and existing Bank account
    /// </remarks>
    /// <param name="accountKey">the Account Number</param>
    /// <returns>true if Account deleted, false if not</returns>
    bool delete (int accountKey);
  }

  ///////////////////////////////////////////////////////////////
  /// <summary>
  /// AccountManager used to create new Accounts or delete accounts.
  /// </summary>
  ///////////////////////////////////////////////////////////////
  
  /// Specify a name for your serviced component
  [ ProgId( "COM+ Bank Server Account Manager" ) ]
  /// Add content to hosting COM+ App's description field
  [ Description( "COM+ Bank Server Account Manager" ) ]
  /// Configure component's Transaction Option
  [ Transaction( TransactionOption.Required ) ]
  /// Configure component's object pooling
  [ ObjectPooling( MinPoolSize = 5, MaxPoolSize = 10, CreationTimeout = 20 ) ]
  /// Specify COM+ Context Attributes
  [ MustRunInClientContext( false ) ]
  /// Enable event tracking
  [ EventTrackingEnabled( true ) ]
  /// Enable JITA for the component
  [ JustInTimeActivation( true ) ]
  /// Enable Construction String Support for the component
  [ ConstructionEnabled( Enabled=true, Default="Gopalan's Bank Server" ) ]
  /// Configure activity-based Synchronization for the component
  [ Synchronization( SynchronizationOption.Required ) ]
  /// Indicate the type of class interface that will be generated for this class
  [ ClassInterface( ClassInterfaceType.AutoDual ) ]

  public class AccountManager : ServicedComponent, IAccountManager {

    /// <summary>
    /// Public No-argument Default Constructor
    /// </summary>
    public AccountManager() {
      MessageBox.Show ("Bank::AccountManager() invoked...");
    }
  
    /////////////////////////////////////////////////////////////////////////
    /// The Following methods support core functionality required of the
    /// AccountManager component and implement the IAccountManager interface
    /////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// The create method
    /// </summary>
    /// <remarks>
    /// Method used to create a new Bank account
    /// </remarks>
    /// <param name="type">Either Checking or Savings</param>
    /// <param name="customerNames">Customers who own this account</param>
    /// <param name="startingBalance">Initial Deposit</param>
    /// <returns></returns>
    
    /// Add content to hosting COM+ App's description field
    [ Description( "Creates a new account for the Bank Server" ) ]

    public int create (AccountType type, string[] customerNames, float startingBalance) {
      MessageBox.Show ("Bank::create() invoked...");
      AccountKey key = null;
      try {
        // Create the BookKeeper class
        BookKeeper.BookKeeper keeper = new BookKeeper.BookKeeper();
        if (null != keeper) {
          // Call the BookKeeper to create a new Bank Account
          key = keeper.createAccount (type, customerNames, startingBalance);
          // Clean-up the BookKeeper object
          keeper.Dispose();
        }
        else {
          MessageBox.Show ("BookKeeper Object is null", "Bank::create()");
          throw new Exception ( "BookKeeper Object could not be created." ); 
        }
        // Since everything went well, commit the changes
        ContextUtil.SetComplete();
      }
      catch (Exception exception) {
        // An Error occured, so rollback the changes
        ContextUtil.SetAbort();

        // Trace the current COM+ context ID (its GUID) to the output window
        // use the ContextId static property of ContextUtil
        Guid contextID = ContextUtil.ContextId;
        String traceMessage = "Context ID is " + contextID.ToString();
        Trace.WriteLine (traceMessage.ToString ());

        MessageBox.Show (exception.ToString (), "Bank::create()");
      }
      return key.Key;
    }

    /// <summary>
    /// The delete method
    /// </summary>
    /// <remarks>
    /// Method used to delete and existing Bank account
    /// </remarks>
    /// <param name="accountKey">the Account Number</param>
    /// <returns>true if Account deleted, false if not</returns>
    
    /// Take advantage of COM+'s method auto-deactivation
    [ AutoComplete( true ) ]
      /// Add content to hosting COM+ App's description field
    [ Description( "Deletes an existing account from the Bank Server" ) ]

    public bool delete (int accountKey) {
      bool result = false;
      MessageBox.Show ("Bank::delete() invoked...", "Key Value = "+accountKey);
      /*
      try {
        
        // Programming Role based security
        SecurityCallContext callContext;
        callContext = SecurityCallContext.CurrentCall;
        string caller = callContext.DirectCaller.AccountName;
        bool isInRole = callContext.IsCallerInRole ( "Manager" );
        if (false == isInRole) {
          throw new Exception ( "Only Managers can delete Customers" );
        }
        */
        BookKeeper.BookKeeper keeper = new BookKeeper.BookKeeper();
        if (null != keeper) {
          AccountKey key = new AccountKey();
          key.Key = accountKey;
          result = keeper.deleteAccount (key);
          keeper.Dispose();
        }
        else {
          MessageBox.Show ("BookKeeper Object is null", "Bank::delete()");
          throw new Exception ( "BookKeeper Object could not be created." ); 
        }
      /*
      }
      catch (Exception exception) {
        Guid contextID = ContextUtil.ContextId;
        String traceMessage = "Context ID is " + contextID.ToString();
        Trace.WriteLine (traceMessage.ToString ());

        MessageBox.Show (exception.ToString (), "Bank::delete()");
      }
      */
      return result;
    }

    /////////////////////////////////////////////////////////////////////////
    /// All the Following methods support overriding functionality required for 
    /// implementing some of the COM+ support interfaces like IObjectConstruct.
    /////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// The method is called after the component's constructor
    /// and is passed in the user specific constructionString
    /// </summary>
    /// <param name="constructionString"></param>
    override public void Construct (string constructionString) {
      MessageBox.Show ("Bank::Construct() invoked...");
      MessageBox.Show (constructionString, "Construction String");
    }

    /// <summary>
    /// Do context specific initialization in this method
    /// </summary>
    override public void Activate () {
      MessageBox.Show ("Bank::Activate() invoked...");
    }

    /// <summary>
    /// Do context specific cleanup in this method
    /// </summary>
    override public void Deactivate () {
      MessageBox.Show ("Bank::Deactivate() invoked...");
    }

    /// <summary>
    /// Object Pooling support method
    /// </summary>
    /// <returns>true if pooling is supported, false if not</returns>
    override public bool CanBePooled () {
      MessageBox.Show ("Bank::CanBePooled() invoked...");
      return true;
    }
  }
}
