//////////////////////////////////////////////////////
/// The following example shows a Distributed Component
/// developed using C# and the .NET Framework.
/// 
/// author: Gopalan Suresh Raj
/// Copyright (c), 2001. All Rights Reserved.
/// URL: http://gsraj.tripod.com/
/// email: gopalan@gmx.net
/// 
//////////////////////////////////////////////////////
using System;
// Include the following to use the Remoting API
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;
// Include the following for the Window Message Box
using System.Windows.Forms; 
// Include the following for ADO.NET functionality
using System.Data;
// Include the BookKeeper namespace
using BookKeeper;

namespace BookKeeper {

  /// <summary>
  /// The AccountDetails structure
  /// </summary>
  
  /// Make this structure serializable
  [Serializable]
  public struct AccountDetails {
    public AccountKey key;
    public string customerNames;
    public float balance;
  }

  /// <summary>
  /// The BookKeeper Class.
  /// </summary>
  
  /// Make this class serializable
  [Serializable]
  public class BookKeeper : MarshalByRefObject, IDisposable {

    int nextKey_;
    DataSet dataSet_;
    bool isDisposed_ = false;

    /// <summary>
    /// Public No-argument Default Constructor
    /// </summary>
    public BookKeeper () {
      // Construct a new DataSet object
      dataSet_ = new DataSet ("BookKeeper");
      try {
        // Populate the object with the contents of the XML File which 
        // contains its schema and data from a previous invocation.
        dataSet_.ReadXml ("Accounts.xml");
      }
      catch (Exception exception) {
        Console.WriteLine (exception.ToString ());
        // If XML File is not found, the schema has not been
        // created yet. So create a schema.
        this.createSchema ();
        MessageBox.Show ("BookKeeper::BookKeeper() created Schema", "BookKeeper");
      }
      nextKey_ = getNextKey ();
    }

    /// <summary>
    /// The Destructor method
    /// </summary>
    ~BookKeeper () {
      if (false == this.isDisposed_) {
        Dispose ();
      }
    }

    /// <summary>
    /// The Dispose method
    /// </summary>
    public void Dispose () {
      this.writeDataSet ();
      MessageBox.Show ("BookKeeper::Dispose() wrote Schema to disk", "BookKeeper");
      // No need to finalize if user called Dispose()
      // so suppress finalization
      GC.SuppressFinalize (this);
      this.isDisposed_ = true;
    }


    /// <summary>
    /// The accessor for the DataSet member
    /// </summary>
    public DataSet dataSet {
      get {
        return this.dataSet_;
      }
    }

    /// <summary>
    /// The createAccount method
    /// </summary>
    /// <param name="accountType">Specifies Checking or Savings account</param>
    /// <param name="customerNames">The Customer Names, owners of this account</param>
    /// <param name="startingBalance">The tarting balance</param>
    /// <returns></returns>
    public AccountKey createAccount (AccountType accountType,
      string[] customerNames, 
      float startingBalance) {
      MessageBox.Show ("BookKeeper::createAccount() invoked", "BookKeeper");
      AccountKey key = null;
      key = new AccountKey ();
      key.Key = nextKey_++;
      key.Type = accountType;
      // Concatenate all customer names
      string names = "";
      foreach (string element in customerNames) {
        names += (" "+element);
      }
      int type = 1;
      if (key.Type == AccountType.SavingsAccount) {
        type = 2;
      }
      this.insertData (key.Key, type, names.Trim(), startingBalance);
      MessageBox.Show ("Key is :"+key.Key, "BookKeeper");

      return key;
    }

    /// <summary>
    /// The deleteAccount method
    /// </summary>
    /// <param name="key">The Account Number to delete</param>
    public bool deleteAccount (AccountKey key) {
      MessageBox.Show ("BookKeeper::deleteAccount() with Key :"+key.Key, "BookKeeper");
      return this.removeAccount (key.Key);
    }

    /// <summary>
    /// The createSchema method
    /// </summary>
    protected void createSchema () {
      MessageBox.Show ("BookKeeper::createSchema() invoked", "BookKeeper");
      lock (this) {
        // Add a new Table named "Accounts" to the DataSet collection tables
        dataSet_.Tables.Add ("Accounts");
        // Add new columns to the table "Accounts"
        dataSet_.Tables ["Accounts"].Columns.Add ("AccountKey",
          Type.GetType ("System.Int32"));
        dataSet_.Tables ["Accounts"].Columns.Add ("AccountType",
          Type.GetType ("System.Int32")); 
        dataSet_.Tables ["Accounts"].Columns.Add ("CustomerNames", 
          Type.GetType ("System.String"));
        dataSet_.Tables ["Accounts"].Columns.Add ("Balance",
          Type.GetType ("System.Currency"));

        // Register the column "AccountKey" as the primary key of the table "Accounts"
        DataColumn[] keys = new DataColumn [1];
        keys [0] = dataSet_.Tables ["Accounts"].Columns ["AccountKey"];
        dataSet_.Tables ["Accounts"].PrimaryKey = keys;
      }
    }

    /// <summary>
    /// The insertData method
    /// </summary>
    /// <param name="accountKey">The Account Number to create</param>
    /// <param name="accountType">Specifies Checking or Savings account</param>
    /// <param name="customerNames">The Customer Names, owners of this account</param>
    /// <param name="balance">The tarting balance</param>
    protected void insertData(int accountKey, 
      int accountType,
      string customerNames,
      float balance) {
      MessageBox.Show ("BookKeeper::insertData() invoked", "BookKeeper");
      lock (this) {
        try {
          DataRow newRow = dataSet_.Tables ["Accounts"].NewRow () ;
          newRow ["AccountKey"] = accountKey;
          newRow ["AccountType"] = accountType;
          newRow ["CustomerNames"] = customerNames;
          newRow ["Balance"] = balance.ToString();
          dataSet_.Tables ["Accounts"].Rows.Add (newRow);
        }
        catch (Exception exception) {
          Console.WriteLine (exception.ToString ());
          MessageBox.Show ("BookKeeper::insertData() Failed", "BookKeeper");
        }
      }
    }

    /// <summary>
    /// The findAccount method
    /// </summary>
    /// <param name="accountKey">The Account Number</param>
    /// <returns>The Details of this account</returns>
    protected AccountDetails findAccount (int accountKey) {
      MessageBox.Show ("BookKeeper::findAccount() invoked", "BookKeeper");
      AccountDetails details = new AccountDetails ();
      
      lock (this) {

        DataTable table = dataSet_.Tables ["Accounts"];
        // Find an order from the table
        DataRow row = table.Rows.Find (accountKey);
      
        // Populate the details object
        AccountKey key = new AccountKey ();
        key.Key = accountKey;
        int type = (int)System.Int32.Parse(row ["AccountType"].ToString ());
        if (type == 1) {
          key.Type = AccountType.CheckingAccount;
        }
        else {
          key.Type = AccountType.SavingsAccount;
        }
        details.key = key;
        details.customerNames = row ["CustomerNames"].ToString ();
        details.balance = (float)System.Double.Parse(row ["Balance"].ToString ());
      }
      return details;
    }

    /// <summary>
    /// The removeAccount method
    /// </summary>
    /// <param name="accountKey">The Account Number</param>
    /// <returns>true if successful, false if not</returns>
    protected bool removeAccount (int accountKey) {
      MessageBox.Show ("BookKeeper::removeAccount() invoked with key: "+accountKey, "BookKeeper");
      bool result = false;
      lock (this) {
        DataTable table = dataSet_.Tables ["Accounts"];
        try {

          table.Rows.Find (accountKey).Delete ();
          table.AcceptChanges ();
          dataSet_.AcceptChanges ();
          result = true;
        }
        catch (Exception exception) {
          Console.WriteLine (exception.ToString ());
          MessageBox.Show (exception.ToString (), "BookKeeper::removeAccount");
        }
      }
      return result;
    }

    /// <summary>
    /// The getNextKey method
    /// </summary>
    /// <returns>The New Account Number (primary key)</returns>
    protected int getNextKey () {
      int result = 1;
      lock (this) {
        try {
          DataTable table = dataSet_.Tables ["Accounts"];
          // This is a hack. But what the heck!
          // This is just a demo !!!
          if (null != table) {
            result = table.Rows.Count+1;
          }
        }
        catch (Exception exception) {
          Console.WriteLine (exception.ToString ());
        }
        finally {
          result = (result == 0)?1:result;
        }
      }
      return result;
    }

    /// <summary>
    /// The writeDataSet method
    /// </summary>
    public void writeDataSet () {
      MessageBox.Show ("BookKeeper::writeDataSet() invoked", "BookKeeper");
      lock (this) {
        dataSet_.AcceptChanges();
        // Dump the DataSet to the Console
        dataSet_.WriteXml (Console.Out, XmlWriteMode.WriteSchema);
        // Dump the DataSet to an XML File
        dataSet_.WriteXml ("Accounts.xml", XmlWriteMode.WriteSchema);
      }
    }

    /// <summary>
    /// The Main method
    /// </summary>
    /// <param name="args"></param>
    static void Main(string[] args) {
      TcpChannel channel = new TcpChannel (1099);
      ChannelServices.RegisterChannel (channel);

      RemotingConfiguration.RegisterWellKnownServiceType (typeof(BookKeeper),
                                                          "BookKeeper",
                                                          WellKnownObjectMode.Singleton);
      System.Console.WriteLine ("Press <Enter> to Exit ...");
      // Instruct the runtime to Garbage Collect
      GC.Collect ();
      System.Console.ReadLine (); 
    }
  }
}
