//////////////////////////////////////////////////////
/// The following example illustrates a Classic COM Type
/// developed using C# and the .NET Framework.
/// 
/// author: Gopalan Suresh Raj
/// Copyrights (c), 2001. All Rights Reserved.
/// URL: http://gsraj.tripod.com/
/// email: gopalan@gmx.net
/// 
/// Here's how to build the assemply
/// <compile>
/// csc /t:library /out:QuoteServer.dll QuoteServer.cs
/// </compile>
/// 
/// Here's how to generate the Type Library and 
/// Register the .NET Types
/// <register>
/// regasm QuoteServer.dll /tlb:QuoteServer.tlb OR
/// tlbexp QuoteServer.dll /out:QuoteServer.tlb
/// </register>
//////////////////////////////////////////////////////

namespace SimpleStocks {

  using System;
  using System.Runtime.InteropServices;
  
  //////////////////////////////////////////////////////
  /// <summary>IQuoteServer Interface</summary>
  /// <remarks>
  /// This interface allows for getting Stock Quotes
  /// </remarks>
  //////////////////////////////////////////////////////
  [ InterfaceTypeAttribute( ComInterfaceType.InterfaceIsDual ) ]
  public interface IQuoteServer {

    //////////////////////////////////////////////////////
    /// <summary>The getQuote method</summary>
    /// <remarks>
    /// Given a Stock Symbol, this method returns
    /// the Stock Quote for that symbol.
    /// </remarks>
    /// <param name="symbol">
    /// The String representing the Symbol for which the 
    /// Stock Quote is requested.
    /// </param>
    /// <returns>
    /// The Price of the Stock corresponding to the symbol
    /// </returns>
    //////////////////////////////////////////////////////
    [ DispId( 1000 ) ] float getQuote( String symbol );
  }

  //////////////////////////////////////////////////////
  /// <summary>QuoteServer Class</summary>
  /// <remarks>
  ///  This Class implements the IQuoteServer interface
  ///  that allows for getting Stock Quotes 
  /// </remarks>
  //////////////////////////////////////////////////////
  [ ClassInterface( ClassInterfaceType.AutoDual ) ]
  public class QuoteServer : IQuoteServer {

    /// <summary>
    /// Public Default Constructor
    /// </summary>
    public QuoteServer () {
    }

    //////////////////////////////////////////////////////
    /// <summary>getQuote Method</summary>
    /// <remarks>
    /// Given a Stock Symbol, this method returns
    /// the Stock Quote for that symbol.
    /// </remarks>
    /// <param name="symbol">
    /// The String representing the Symbol for which the 
    /// Stock Quote is requested.
    /// </param>
    /// <returns>
    /// The Price of the Stock corresponding to the symbol
    /// </returns>
    /// <exception cref="ArgumentNullException"></exception>
    //////////////////////////////////////////////////////
    public float getQuote ( String symbol ) {
      if (null == symbol) {
        // Intercepted as a COM Error Object
        throw new ArgumentNullException(symbol);
      }

      float price = 0;
      char[] symbolArray = symbol.ToCharArray ();
      for (int index = 0; index < symbolArray.Length; index++) {
        price += (int) symbolArray [index];
      }
      price /= 5;
      return price;
    }

    //////////////////////////////////////////////////////
    /// <summary>addExtraRegistrationLogic method</summary>
    /// <remarks>
    /// This attribute configures this method to be invoked during
    /// registration of this assembly
    /// </remarks>
    //////////////////////////////////////////////////////
    [ ComRegisterFunctionAttribute ]
    public static void addExtraRegistrationLogic(string registrationLogic) {
      // perform any extra logic when registration occurs.
      Console.WriteLine( "Invoked by the system since Registration is occuring ..." );
    }

    //////////////////////////////////////////////////////
    /// <summary>removeExtraRegistrationLogic method</summary>
    /// <remarks>
    /// This attribute configures this method to be invoked during
    /// unregistration of this assembly
    /// </remarks>
    //////////////////////////////////////////////////////
    [ ComUnregisterFunctionAttribute ]
    public static void removeExtraRegistrationLogic(string registrationLogic) {
      // perform any extra logic when unregistration occurs.
      Console.WriteLine( "Invoked by the system since Unregistration is occuring ..." );
    }

  }
}